<?php

namespace WHMCS\Module\Server\VirtFusionDirect;

use WHMCS\Database\Capsule as DB;
use WHMCS\Module\Server\VirtFusionDirect\Curl;

class Module
{
    protected $dbTable = 'mod_virtfusion_direct';

    public function __construct()
    {
        error_reporting(0);
        $this->dbSchema();
    }

    private function dbSchema()
    {
        if (!DB::schema()->hasTable($this->dbTable)) {
            try {
                DB::schema()->create($this->dbTable, function ($table) {
                    $table->unsignedBigInteger('service_id')->nullable()->default(null)->index();
                    $table->unsignedBigInteger('server_id')->nullable()->default(null);
                    $table->timestamps();
                });
            } catch (\Exception $e) {
                $this->log( __FUNCTION__, [], $e->getMessage());
                echo "Unable to create database table: {$e->getMessage()}";
            }
        }
    }

    /**
     * @param bool $exitOnError
     * @return mixed
     */
    public function validateAction($exitOnError = true)
    {
        if (!isset($_GET['action'])) {
            $this->output(['errors' => 'no action specified'], true, $exitOnError, 200);
        }
        return $_GET['action'];
    }

    /**
     * @param bool $exitOnError
     * @return mixed
     */
    public function validateServiceID($exitOnError = true)
    {
        if (!isset($_GET['serviceID'])) {
            $this->output(['errors' => 'no serviceID specified'], true, $exitOnError, 200);
        }
        return $_GET['serviceID'];
    }

    /**
     * @param $serviceID
     * @param bool $exitOnError
     * @return bool
     */
    public function validateUserOwnsService($serviceID, $exitOnError = true)
    {
        $currentUser = new \WHMCS\Authentication\CurrentUser;
        $client = $currentUser->client();

        if (!$client) {
            return false;
        }

        if (DB::table('tblhosting')->where('id', (int)$serviceID)->where('userid', $client->id)->exists()) {
            return true;
        }
        return false;
    }

    /**
     * @param $serviceID
     * @return false|string
     */
    public function fetchLoginTokens($serviceID)
    {
        $service = DB::table($this->dbTable)->where('service_id', $serviceID)->first();

        if ($service) {
            $whmcsService = DB::table('tblhosting')->where('id', $serviceID)->first();
            $cp = $this->getCP($whmcsService->server);
            $request = $this->initCurl($cp['token']);
            $data = $request->post($cp['url'] . '/users/' . $whmcsService->userid . '/serverAuthenticationTokens/' . $service->server_id);

            $this->log( __FUNCTION__, $request->getRequestInfo(), $data);
            if ($request->getRequestInfo('http_code') == '200') {
                $data = json_decode($data);
                return $cp['base_url'] . $data->data->authentication->endpoint_complete;
            }
        }
        return false;
    }

    /**
     * @param $data
     * @param bool $json
     * @param bool $exit
     * @param int $rspCode
     */
    public function output($data, $json = true, $exit = true, $rspCode = 200)
    {
        http_response_code($rspCode);

        if ($json) {
            header('Content-Type: application/json; charset=utf-8');
            echo json_encode($data);
        } else {
            echo $data;
        }

        if ($exit) {
            exit();
        }
    }

    /**
     * @param $server
     * @return array|false
     */
    protected function getCP($server)
    {
        if ($server) {
            $cp = DB::table('tblservers')->where('type', 'VirtFusionDirect')->where('id', $server)->first();
        } else {
            $cp = DB::table('tblservers')->where('type', 'VirtFusionDirect')->where('disabled', 0)->first();
        }

        if ($cp) {
            return [
                'url' => 'https://' . $cp->hostname . '/api/v1',
                'base_url' => 'https://' . $cp->hostname,
                'token' => decrypt($cp->password)];
        }
        return false;
    }

    /**
     * @return bool|void
     */
    public function adminOnly()
    {
        if ((new \WHMCS\Authentication\CurrentUser)->isAuthenticatedAdmin()) {
            return true;
        }

        $this->output(['errors' => 'unauthenticated'], true, true, 200);
    }

    /**
     * @return bool|void
     */
    public function isAuthenticated()
    {
        if ((new \WHMCS\Authentication\CurrentUser)->isAuthenticatedUser()) {
            return true;
        }

        $this->output(['errors' => 'unauthenticated'], true, true, 200);
    }

    /**
     * @param $serviceId
     * @return mixed
     */
    protected function serviceExistsInCustomTable($serviceId)
    {
        return DB::table($this->dbTable)->where('service_id', $serviceId)->exists();
    }

    /**
     * @param $serviceId
     */
    protected function deleteServiceFromCustomTable($serviceId)
    {
        DB::table($this->dbTable)->where('service_id', $serviceId)->delete();
    }

    /**
     * @param $token
     * @return \WHMCS\Module\Server\VirtFusionDirect\Curl
     */
    protected function initCurl($token)
    {
        $curl = new Curl();

        $curl->addOption(CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Content-type: application/json; charset=utf-8',
            'authorization: Bearer ' . $token
        ]);

        return $curl;
    }

    /**
     * @param $function
     * @param $data1
     * @param $data2
     */
    protected function log($function, $data1, $data2)
    {
        logModuleCall('VirtFusionDirect', $function, $data1, $data2);
    }
}